@@ -109,22 +109,30 @@ sub new {
 
     $self->outgoing_proxy($outgoing_proxy) if $outgoing_proxy;
 
-    # regularly prune queue
-    my $sub = sub {
-	my ($self,$timer) = @_;
-	if ( $self ) {
-	    $self->queue_expire( $self->{eventloop}->looptime );
-	} else {
-	    $timer->cancel;
-	}
-    };
-    my $cb = [ $sub,$self ];
-    weaken( $cb->[1] );
-    $self->{disp_expire} = $self->add_timer( 1,$cb,1,'disp_expire' );
-
     return $self;
 }
 
+# regularly prune queue
+sub __disp_expire_timer {
+    my $self = shift || return; # dispatcher already deleted (weak reference)
+    my $min_expire = $self->queue_expire($self->{eventloop}->looptime);
+    if (!$min_expire) {
+	# nothing in queue to expire
+	delete $self->{disp_expire};
+	return;
+    }
+    # add timer again
+    $self->{disp_expire} = $self->add_timer(
+	1, [\&__disp_expire_timer, $self], undef, 'disp_expire');
+}
+sub __set_disp_expire_timer {
+    my Net::SIP::Dispatcher $self = shift;
+    $self->{disp_expire} and return;
+    my $cb = [\&__disp_expire_timer, $self];
+    weaken($cb->[1]);
+    $self->{disp_expire} = $self->add_timer(1, $cb, undef, 'disp_expire');
+}
+
 ###########################################################################
 # get or set outgoing proxy
 # Args: ($self;$proxy)
@@ -413,6 +421,7 @@ sub deliver {
     $new_entry->prepare_retransmits( $now ) if $do_retransmits;
 
     push @{ $self->{queue}}, $new_entry;
+    $self->__set_disp_expire_timer;
     $self->__deliver( $new_entry );
 }
 
@@ -459,6 +468,7 @@ sub cancel_delivery {
     } else {
 	croak( "cancel_delivery w/o id" );
     }
+    $self->__set_disp_expire_timer;
     return @$q < $qn; # true if items got deleted
 }
 
@@ -563,6 +573,7 @@ sub queue_expire {
     }
 
     # return time to next expire for optimizations
+    DEBUG( 50,"next expire %s", $min_expire || '<undef>' );
     return $min_expire;
 }
 
@@ -682,6 +693,7 @@ sub resolve_uri {
     };
 
     my @proto;
+    my $force_port;
     my $default_port = 5060;
     if ( $sip_proto eq 'sips' ) {
 	$default_port = 5061;
@@ -724,8 +736,12 @@ sub resolve_uri {
 
     my $ip_addr = $param->{maddr};
     {
-	my ($host,$port,$family) = ip_string2parts($domain, $ip_addr ? 1:0);
-	$default_port = $port if defined $port;
+	my ($host,$port,$family) = eval { ip_string2parts($domain, $ip_addr ? 1:0) };
+	$host or do {
+	    DEBUG( 50,"bad URI '$uri'" );
+	    return invoke_callback($callback, EHOSTUNREACH );
+	};
+	$force_port = $port if defined $port;
 	if ($family) {
 	    $ip_addr ||= $host;
 	    $domain = ip_ptr($host,$family);
@@ -749,6 +765,7 @@ sub resolve_uri {
 	    if ( $addr ) {
 		DEBUG( 50,"setting dst_addr from domain specific proxy for domain $dom" );
 		@$dst_addr = @$addr;
+		undef $force_port;
 	    }
 	}
     }
@@ -759,6 +776,7 @@ sub resolve_uri {
 	# if we have a fixed outgoing proxy use it
 	DEBUG( 50,"setting dst_addr+leg to $addr from outgoing_proxy" );
 	@$dst_addr = ( $addr );
+	undef $force_port;
     }
 
     # is it an IP address?
@@ -785,7 +803,7 @@ sub resolve_uri {
 		proto  => $proto,
 		host   => $host,
 		addr   => $family ? $host : undef,
-		port   => $port || $default_port,
+		port   => $port || $force_port || $default_port,
 		family => $family
 	    });
 	    push @resp, map { lock_ref_keys({
@@ -813,7 +831,7 @@ sub resolve_uri {
 	return invoke_callback($callback, ENOPROTOOPT) if ! @resp;
     }
 
-    my @param = ( $dst_addr,$legs,$allowed_legs,$default_port,$callback );
+    my @param = ( $dst_addr,$legs,$allowed_legs,$force_port,$default_port,$callback );
     if (@resp) {
 	# directly call __resolve_uri_final if all names are resolved
 	return __resolve_uri_final( @param,\@resp )
@@ -828,43 +846,69 @@ sub resolve_uri {
     # but query instead directly for _sip._udp.domain.. like in
     # RFC2543 specified
 
+    # filter protocols not supported by any leg
+    my @proto_new;
+    foreach my $p ( @proto ) {
+	my $l = first { $_->match({ proto => $p }) } @$allowed_legs;
+	push @proto_new,$p if $l;
+    }
+    @proto = @proto_new;
+    @proto or do {
+	DEBUG( 50,"no legs allowed for $uri" );
+	@$dst_addr = ();
+	return invoke_callback( $callback, ENOPROTOOPT ); # no proto available
+    };
+
     return $self->dns_domain2srv($domain, \@proto,
 	[ \&__resolve_uri_final, @param ]);
 }
 
 sub __resolve_uri_final {
-    my ($dst_addr,$legs,$allowed_legs,$default_port,$callback,$resp) = @_;
+    my ($dst_addr,$legs,$allowed_legs,$force_port,$default_port,$callback,$resp) = @_;
     $DEBUG && DEBUG_DUMP( 100,$resp );
 
     return invoke_callback( $callback,EHOSTUNREACH )
 	unless $resp && @$resp;
 
+    # overwrite port if it was forced by user
+    do { $_->{port} = $force_port for(@$resp) } if $force_port;
+
     # for A|AAAA records we got no port, use default_port
     $_->{port} ||= $default_port for(@$resp);
 
-    # sort by prio
+    # sort by prio and eliminate duplicates
     # FIXME: can contradict order in @proto
-    @$resp = sort { $a->{prio} <=> $b->{prio} } @$resp;
+    if (@$resp>1) {
+	my %dup;
+	@$resp =
+	    sort { $a->{prio} <=> $b->{prio} }
+	    grep { !$dup{$_->{host}}{$_->{family}}{$_->{proto}}{$_->{addr}}{$_->{port}}++ }
+	    @$resp;
+    }
 
     @$dst_addr = ();
     @$legs = ();
     foreach my $r ( @$resp ) {
-	my $leg = first { $_->can_deliver_to(
+	if (my @l = grep { $_->can_deliver_to(
 	    proto  => $r->{proto},
 	    host   => $r->{host},
 	    addr   => $r->{addr},
 	    port   => $r->{port},
 	    family => $r->{family},
-	)} @$allowed_legs;
-
-	if ( $leg ) {
+	)} @$allowed_legs) {
 	    push @$dst_addr, $r;
-	    push @$legs,$leg;
+	    push @$legs, @l;
 	} else {
 	    DEBUG(50,"no leg with $r->{proto} to %s", ip_parts2string($r));
 	}
     }
 
+    # remove duplicates
+    if (@$legs>1) {
+	my %dup;
+	@$legs = grep { !$dup{$_}++ } @$legs;
+    }
+
     return invoke_callback( $callback, EHOSTUNREACH ) if !@$dst_addr;
     invoke_callback( $callback );
 }
@@ -1048,7 +1092,7 @@ sub __generic_resolver {
     for(my $i=0; $i<@$queries; $i++) {
 	my $q = $queries->[$i];
 	if ($q->{type} eq 'BREAK-IF-RESULTS') {
-	    if (@$results) {
+	    if (@$results and $i==0) {
 		# skip remaining queries
 		@$queries = ();
 		last;
@@ -1105,7 +1149,7 @@ sub __generic_resolver {
 		$i--;
 	    }
 	    for my $r (@res) {
-		if ($_->{family}) {
+		if ($r->{family}) {
 		    # done: host in SRV record is already IP address
 		    push @$results, $r;
 		    next;
@@ -1170,6 +1214,7 @@ sub __net_dns_resolver {
 	my ($sock,$callback) = @_;
 	my $q = $NetDNSResolver->bgread($sock);
 	$eventloop->delFD($sock);
+	close($sock);
 	my @ans;
 	for my $rr ( $q->answer ) {
 	    if ($rr->type eq 'SRV' ) {
