@@ -165,10 +165,12 @@ sub new {
 	    $self->{src} = ip_sockaddr2parts($saddr);
 	    $self->{src}{host} = $host if $host;
 	}
-	if ((!$dst or !$sockpeer) and my $saddr = getpeername($sock)) {
-	    # set from connected socket
-	    $sockpeer = ip_sockaddr2parts($saddr);
-	    $dst ||= $sockpeer;
+	if (my $saddr = getpeername($sock)) {
+	    if (!$dst) {
+		# set dst from connected socket
+		$dst = ip_sockaddr2parts($saddr);
+	    }
+	    $sockpeer = $dst;
 	}
     }
 
